import 'reflect-metadata';
import { Property, PropertyDefinition } from '../types/property';

/**
 * Register an exposed property. This is displayed in the editor and serialized to files.
 * @param editorPropertyDefinition Additional options
 */
export function ExposeProperty(
  editorPropertyDefinition?: Omit<PropertyDefinition, 'name'>
): PropertyDecorator {
  return (target, propertyKey): void => {
    let properties: Array<Property> = Reflect.getOwnMetadata(
      'properties',
      target
    );

    if (!properties) {
      Reflect.defineMetadata('properties', (properties = []), target);
    }

    properties.push(
      new Property({ ...editorPropertyDefinition, name: String(propertyKey) })
    );
  };
}

/**
 * Retract a property from an ancestor ExposedProperty list
 */
export function RetractProperty(): PropertyDecorator {
  return (target, propertyKey): void => {
    let excluded: Array<string> = Reflect.getOwnMetadata(
      'excludedProperties',
      target
    );

    if (!excluded) {
      Reflect.defineMetadata('excludedProperties', (excluded = []), target);
    }

    excluded.push(String(propertyKey));
  };
}
