import { PhysicsObject } from './physics.object';
import { gameObjectGeometries } from './geometries';
import { BufferGeometry, NormalBufferAttributes, Vector3 } from 'three';
import type Rapier from '@dimforge/rapier3d';

export class Brick extends PhysicsObject {
  protected geometry?: BufferGeometry<NormalBufferAttributes> | undefined =
    gameObjectGeometries.boxGeometry;
  public objectType = 'Brick';
  public name = 'Brick';

  protected override createCollider(
    factory: typeof Rapier,
    world: Rapier.World,
    body?: Rapier.RigidBody
  ) {
    const scale = this.getWorldScale(new Vector3()).divideScalar(2);
    const collider = factory.ColliderDesc.cuboid(
      Math.abs(scale.x),
      Math.abs(scale.y),
      Math.abs(scale.z)
    );
    return world.createCollider(collider, body);
  }
}
