import { Color, Vector3 } from 'three';
import { GameObject, SerializedObject } from '../types/game-object';
import { environmentDefaults } from '../defaults/environment';
import { ExposeProperty, RetractProperty } from '../decorators/property';

export class Environment extends GameObject {
  public objectType = 'Environment';
  @RetractProperty()
  public name = 'Environment';
  public virtual = true;

  @RetractProperty()
  public override visible!: boolean;

  @ExposeProperty({ type: Color })
  sunColor = environmentDefaults.sunColor.clone();

  @ExposeProperty({ type: Vector3 })
  sunPosition = environmentDefaults.sunPosition.clone();

  @ExposeProperty({ type: Number })
  sunStrength = environmentDefaults.sunStrength;

  @ExposeProperty({ type: Color })
  ambientColor = environmentDefaults.ambientColor.clone();

  @ExposeProperty({ type: Number })
  ambientStrength = environmentDefaults.ambientStrength;

  @ExposeProperty({ type: Color })
  clearColor = environmentDefaults.clearColor.clone();

  override serialize() {
    return super.serialize() as SerializedEnvironment;
  }
}

export interface SerializedEnvironment extends SerializedObject {
  sunColor: Color;
  sunPosition: Vector3;
  sunStrength: number;
  ambientColor: Color;
  ambientStrength: number;
  clearColor: Color;
}
