import { MeshPhongMaterial, SkinnedMesh, Vector3 } from 'three';
import { Brick } from '.';
import type Rapier from '@dimforge/rapier3d';

export class MeshPart extends Brick {
  public objectType = 'MeshPart';

  /** Do some surgery to convert a loaded SkinnedMesh into a game object */
  static fromLoaded(loaded: SkinnedMesh) {
    const newObject = new MeshPart();
    const newMaterial = new MeshPhongMaterial();
    newObject.mesh = loaded;
    newObject.mesh.material = newMaterial;
    newObject.material = newMaterial;
    newObject.geometry = loaded.geometry;
    newObject.add(newObject.mesh);
    return newObject;
  }

  protected override createCollider(
    factory: typeof Rapier,
    world: Rapier.World,
    body?: Rapier.RigidBody
  ) {
    const scale = this.getWorldScale(new Vector3()).divideScalar(2);
    const collider = factory.ColliderDesc.cuboid(
      Math.abs(scale.x),
      Math.abs(scale.y),
      Math.abs(scale.z)
    );
    return world.createCollider(collider, body);
  }

  override initializeMesh(): void {
    this.add(this.mesh!);
  }
}
